@@ -2,7 +2,6 @@ use poem::{FromRequest, IntoResponse, Request, RequestBody, Response};
 
 use crate::{
     payload::{ParsePayload, Payload},
-    poem::Error,
     registry::{MetaMediaType, MetaResponse, MetaResponses, MetaSchema, MetaSchemaRef, Registry},
     ApiResponse, ParseRequestError,
 };
@@ -29,14 +28,7 @@ impl ParsePayload for Binary<Vec<u8>> {
         body: &mut RequestBody,
     ) -> Result<Self, ParseRequestError> {
         Ok(Self(<Vec<u8>>::from_request(request, body).await.map_err(
-            |err| {
-                ParseRequestError::ParseRequestBody {
-                    reason: Into::<Error>::into(err)
-                        .reason()
-                        .unwrap_or_default()
-                        .to_string(),
-                }
-            },
+            |err| ParseRequestError::ParseRequestBody(err.into_response()),
         )?))
     }
 }
