@@ -16,7 +16,7 @@ impl Type for Binary {
         "string(binary)".into()
     }
 
-    impl_value_type!();
+    impl_raw_value_type!();
 
     fn schema_ref() -> MetaSchemaRef {
         MetaSchemaRef::Inline(Box::new(MetaSchema::new_with_format("string", "binary")))
