@@ -29,8 +29,15 @@ pub trait Type: Send + Sync {
     /// If it is `true`, it means that this value is required.
     const IS_REQUIRED: bool = true;
 
-    /// The value type of this type.
-    type ValueType;
+    /// The raw type used for validator.
+    ///
+    /// Usually it is `Self`, but the wrapper type is its internal type.
+    ///
+    /// For example:
+    ///
+    /// `i32::RawValueType` is `i32`
+    /// `Option<i32>::RawValueType` is `i32`.
+    type RawValueType;
 
     /// Returns the name of this type
     fn name() -> Cow<'static, str>;
@@ -42,40 +49,31 @@ pub trait Type: Send + Sync {
     #[allow(unused_variables)]
     fn register(registry: &mut Registry) {}
 
-    /// Get the value.
-    fn as_value(&self) -> Option<&Self::ValueType>;
+    /// Returns a reference to the raw value.
+    fn as_raw_value(&self) -> Option<&Self::RawValueType>;
 }
 
 /// Represents a type that can parsing from JSON.
-pub trait ParseFromJSON: Type {
+pub trait ParseFromJSON: Sized + Type {
     /// Parse from [`serde_json::Value`].
-    fn parse_from_json(value: Value) -> ParseResult<Self>
-    where
-        Self: Sized;
+    fn parse_from_json(value: Value) -> ParseResult<Self>;
 }
 
 /// Represents a type that can parsing from parameter. (header, query, path,
 /// cookie)
-pub trait ParseFromParameter: Type {
+pub trait ParseFromParameter: Sized + Type {
     /// Parse from parameter.
-    fn parse_from_parameter(value: Option<&str>) -> ParseResult<Self>
-    where
-        Self: Sized;
+    fn parse_from_parameter(value: Option<&str>) -> ParseResult<Self>;
 }
 
 /// Represents a type that can parsing from multipart.
 #[poem::async_trait]
-pub trait ParseFromMultipartField: Type {
+pub trait ParseFromMultipartField: Sized + Type {
     /// Parse from multipart field.
-    async fn parse_from_multipart(field: Option<PoemField>) -> ParseResult<Self>
-    where
-        Self: Sized;
+    async fn parse_from_multipart(field: Option<PoemField>) -> ParseResult<Self>;
 
     /// Parse from repeated multipart field.
-    async fn parse_from_repeated_field(self, _field: PoemField) -> ParseResult<Self>
-    where
-        Self: Sized,
-    {
+    async fn parse_from_repeated_field(self, _field: PoemField) -> ParseResult<Self> {
         Err(ParseError::<Self>::custom("repeated field"))
     }
 }
