@@ -102,12 +102,16 @@ async fn required_fields() {
     )
     .await
     .unwrap_err();
-    assert_eq!(
-        err,
-        ParseRequestError::ParseRequestBody {
-            reason: "field `file` is required".to_string()
+
+    match err {
+        ParseRequestError::ParseRequestBody(resp) => {
+            assert_eq!(
+                resp.into_body().into_string().await.unwrap(),
+                "field `file` is required"
+            );
         }
-    );
+        _ => panic!(),
+    }
 }
 
 #[tokio::test]
@@ -241,9 +245,9 @@ async fn upload() {
 async fn validator() {
     #[derive(Multipart, Debug, Eq, PartialEq)]
     struct A {
-        #[oai(max_length = "10")]
+        #[oai(validator(max_length = "10"))]
         name: String,
-        #[oai(maximum(value = "32"))]
+        #[oai(validator(maximum(value = "32")))]
         value: JsonField<i32>,
     }
 
@@ -268,13 +272,16 @@ async fn validator() {
     )
     .await
     .unwrap_err();
-    assert_eq!(
-        err,
-        ParseRequestError::ParseRequestBody {
-            reason: r#"field `value` verification failed. maximum(32, exclusive: false)"#
-                .to_string()
+
+    match err {
+        ParseRequestError::ParseRequestBody(resp) => {
+            assert_eq!(
+                resp.into_body().into_string().await.unwrap(),
+                r#"field `value` verification failed. maximum(32, exclusive: false)"#
+            );
         }
-    );
+        _ => panic!(),
+    }
 }
 
 #[tokio::test]
@@ -420,13 +427,16 @@ async fn repeated_error() {
     )
     .await
     .unwrap_err();
-    assert_eq!(
-        err,
-        ParseRequestError::ParseRequestBody {
-            reason: "failed to parse field `value`: failed to parse \"string\": repeated field"
-                .to_string()
+
+    match err {
+        ParseRequestError::ParseRequestBody(resp) => {
+            assert_eq!(
+                resp.into_body().into_string().await.unwrap(),
+                "failed to parse field `value`: failed to parse \"string\": repeated field"
+            );
         }
-    )
+        _ => panic!(),
+    }
 }
 
 #[test]
